@@ -1,4 +1,4 @@
-/* $OpenBSD: tls13_record_layer.c,v 1.33.4.1 2020/08/10 18:59:47 tb Exp $ */
+/* $OpenBSD: tls13_record_layer.c,v 1.47 2020/05/29 17:54:58 jsing Exp $ */
 /*
  * Copyright (c) 2018, 2019 Joel Sing <jsing@openbsd.org>
  *
@@ -30,9 +30,11 @@ struct tls13_record_layer {
 
 	int ccs_allowed;
 	int ccs_seen;
+	int ccs_sent;
 	int handshake_completed;
 	int legacy_alerts_allowed;
 	int phh;
+	int phh_retry;
 
 	/*
 	 * Read and/or write channels are closed due to an alert being
@@ -79,14 +81,8 @@ struct tls13_record_layer {
 	uint8_t read_seq_num[TLS13_RECORD_SEQ_NUM_LEN];
 	uint8_t write_seq_num[TLS13_RECORD_SEQ_NUM_LEN];
 
-	/* Record callbacks. */
-	tls13_alert_cb alert_cb;
-	tls13_phh_recv_cb phh_recv_cb;
-	tls13_phh_sent_cb phh_sent_cb;
-
-	/* Wire read/write callbacks. */
-	tls13_read_cb wire_read;
-	tls13_write_cb wire_write;
+	/* Callbacks. */
+	struct tls13_record_layer_callbacks cb;
 	void *cb_arg;
 };
 
@@ -115,10 +111,7 @@ tls13_record_layer_wrec_free(struct tls13_record_layer *rl)
 }
 
 struct tls13_record_layer *
-tls13_record_layer_new(tls13_read_cb wire_read, tls13_write_cb wire_write,
-    tls13_alert_cb alert_cb,
-    tls13_phh_recv_cb phh_recv_cb,
-    tls13_phh_sent_cb phh_sent_cb,
+tls13_record_layer_new(const struct tls13_record_layer_callbacks *callbacks,
     void *cb_arg)
 {
 	struct tls13_record_layer *rl;
@@ -127,12 +120,7 @@ tls13_record_layer_new(tls13_read_cb wire_read, tls13_write_cb wire_write,
 		return NULL;
 
 	rl->legacy_version = TLS1_2_VERSION;
-
-	rl->wire_read = wire_read;
-	rl->wire_write = wire_write;
-	rl->alert_cb = alert_cb;
-	rl->phh_recv_cb = phh_recv_cb;
-	rl->phh_sent_cb = phh_sent_cb;
+	rl->cb = *callbacks;
 	rl->cb_arg = cb_arg;
 
 	return rl;
@@ -246,6 +234,12 @@ tls13_record_layer_handshake_completed(struct tls13_record_layer *rl)
 	rl->handshake_completed = 1;
 }
 
+void
+tls13_record_layer_set_retry_after_phh(struct tls13_record_layer *rl, int retry)
+{
+	rl->phh_retry = retry;
+}
+
 static ssize_t
 tls13_record_layer_process_alert(struct tls13_record_layer *rl)
 {
@@ -267,13 +261,13 @@ tls13_record_layer_process_alert(struct tls13_record_layer *rl)
 		return TLS13_IO_FAILURE;
 
 	if (!CBS_get_u8(&rl->rbuf_cbs, &alert_level))
-		return tls13_send_alert(rl, TLS1_AD_DECODE_ERROR);
+		return tls13_send_alert(rl, TLS13_ALERT_DECODE_ERROR);
 
 	if (!CBS_get_u8(&rl->rbuf_cbs, &alert_desc))
-		return tls13_send_alert(rl, TLS1_AD_DECODE_ERROR);
+		return tls13_send_alert(rl, TLS13_ALERT_DECODE_ERROR);
 
 	if (CBS_len(&rl->rbuf_cbs) != 0)
-		return tls13_send_alert(rl, TLS1_AD_DECODE_ERROR);
+		return tls13_send_alert(rl, TLS13_ALERT_DECODE_ERROR);
 
 	tls13_record_layer_rbuf_free(rl);
 
@@ -282,24 +276,25 @@ tls13_record_layer_process_alert(struct tls13_record_layer *rl)
 	 * however for error alerts (RFC 8446 section 6.2), the alert level
 	 * must be specified as fatal.
 	 */
-	if (alert_desc == SSL_AD_CLOSE_NOTIFY) {
+	if (alert_desc == TLS13_ALERT_CLOSE_NOTIFY) {
 		rl->read_closed = 1;
 		ret = TLS13_IO_EOF;
-	} else if (alert_desc == SSL_AD_USER_CANCELLED) {
+	} else if (alert_desc == TLS13_ALERT_USER_CANCELED) {
 		/* Ignored at the record layer. */
 		ret = TLS13_IO_WANT_RETRY;
-	} else if (alert_level == SSL3_AL_FATAL) {
+	} else if (alert_level == TLS13_ALERT_LEVEL_FATAL) {
 		rl->read_closed = 1;
 		rl->write_closed = 1;
 		ret = TLS13_IO_ALERT;
-	} else if (rl->legacy_alerts_allowed && alert_level == SSL3_AL_WARNING) {
+	} else if (rl->legacy_alerts_allowed &&
+	    alert_level == TLS13_ALERT_LEVEL_WARNING) {
 		/* Ignored and not passed to the callback. */
 		return TLS13_IO_WANT_RETRY;
 	} else {
-		return tls13_send_alert(rl, SSL_AD_ILLEGAL_PARAMETER);
+		return tls13_send_alert(rl, TLS13_ALERT_ILLEGAL_PARAMETER);
 	}
 
-	rl->alert_cb(alert_desc, rl->cb_arg);
+	rl->cb.alert_recv(alert_desc, rl->cb_arg);
 
 	return ret;
 }
@@ -321,10 +316,10 @@ tls13_record_layer_send_alert(struct tls13_record_layer *rl)
 	rl->alert_data = NULL;
 	rl->alert_len = 0;
 
-	if (rl->alert_desc == SSL_AD_CLOSE_NOTIFY) {
+	if (rl->alert_desc == TLS13_ALERT_CLOSE_NOTIFY) {
 		rl->write_closed = 1;
 		ret = TLS13_IO_SUCCESS;
-	} else if (rl->alert_desc == SSL_AD_USER_CANCELLED) {
+	} else if (rl->alert_desc == TLS13_ALERT_USER_CANCELED) {
 		/* Ignored at the record layer. */
 		ret = TLS13_IO_SUCCESS;
 	} else {
@@ -333,6 +328,8 @@ tls13_record_layer_send_alert(struct tls13_record_layer *rl)
 		ret = TLS13_IO_ALERT;
 	}
 
+	rl->cb.alert_sent(rl->alert_desc, rl->cb_arg);
+
 	return ret;
 }
 
@@ -343,7 +340,7 @@ tls13_record_layer_send_phh(struct tls13_record_layer *rl)
 
 	/* Push out pending post-handshake handshake messages. */
 	if ((ret = tls13_record_layer_write_chunk(rl, SSL3_RT_HANDSHAKE,
-	    CBS_data(&rl->phh_cbs), CBS_len(&rl->phh_cbs))) < 0)
+	    CBS_data(&rl->phh_cbs), CBS_len(&rl->phh_cbs))) <= 0)
 		return ret;
 	if (!CBS_skip(&rl->phh_cbs, ret))
 		return TLS13_IO_FAILURE;
@@ -356,7 +353,7 @@ tls13_record_layer_send_phh(struct tls13_record_layer *rl)
 
 	CBS_init(&rl->phh_cbs, rl->phh_data, rl->phh_len);
 
-	rl->phh_sent_cb(rl->cb_arg);
+	rl->cb.phh_sent(rl->cb_arg);
 
 	return TLS13_IO_SUCCESS;
 }
@@ -435,8 +432,6 @@ tls13_record_layer_set_traffic_key(const EVP_AEAD *aead, EVP_AEAD_CTX *aead_ctx,
 	struct tls13_secret key = { .data = NULL, .len = 0 };
 	int ret = 0;
 
-	EVP_AEAD_CTX_cleanup(aead_ctx);
-
 	freezero(iv->data, iv->len);
 	iv->data = NULL;
 	iv->len = 0;
@@ -525,9 +520,8 @@ static int
 tls13_record_layer_open_record_protected(struct tls13_record_layer *rl)
 {
 	CBS header, enc_record;
-	ssize_t inner_len;
 	uint8_t *content = NULL;
-	size_t content_len = 0;
+	ssize_t content_len = 0;
 	uint8_t content_type;
 	size_t out_len;
 
@@ -554,6 +548,9 @@ tls13_record_layer_open_record_protected(struct tls13_record_layer *rl)
 	    CBS_data(&header), CBS_len(&header)))
 		goto err;
 
+	if (out_len > TLS13_RECORD_MAX_INNER_PLAINTEXT_LEN)
+		goto err;
+
 	if (!tls13_record_layer_inc_seq_num(rl->read_seq_num))
 		goto err;
 
@@ -563,18 +560,20 @@ tls13_record_layer_open_record_protected(struct tls13_record_layer *rl)
 	 * Time to hunt for that elusive content type!
 	 */
 	/* XXX - CBS from end? CBS_get_end_u8()? */
-	inner_len = out_len - 1;
-	while (inner_len >= 0 && content[inner_len] == 0)
-		inner_len--;
-	if (inner_len < 0)
+	content_len = out_len - 1;
+	while (content_len >= 0 && content[content_len] == 0)
+		content_len--;
+	if (content_len < 0)
 		goto err;
-	content_type = content[inner_len];
+	if (content_len > TLS13_RECORD_MAX_PLAINTEXT_LEN)
+		goto err;
+	content_type = content[content_len];
 
 	tls13_record_layer_rbuf_free(rl);
 
 	rl->rbuf_content_type = content_type;
 	rl->rbuf = content;
-	rl->rbuf_len = inner_len;
+	rl->rbuf_len = content_len;
 
 	CBS_init(&rl->rbuf_cbs, rl->rbuf, rl->rbuf_len);
 
@@ -606,7 +605,14 @@ tls13_record_layer_seal_record_plaintext(struct tls13_record_layer *rl,
 	size_t data_len = 0;
 	CBB cbb, body;
 
-	if (rl->aead != NULL)
+	/*
+	 * Allow dummy CCS messages to be sent in plaintext even when
+	 * record protection has been engaged, as long as the handshake
+	 * has not yet completed.
+	 */
+	if (rl->handshake_completed)
+		return 0;
+	if (rl->aead != NULL && content_type != SSL3_RT_CHANGE_CIPHER_SPEC)
 		return 0;
 
 	/*
@@ -755,7 +761,7 @@ tls13_record_layer_seal_record(struct tls13_record_layer *rl,
 	if ((rl->wrec = tls13_record_new()) == NULL)
 		return 0;
 
-	if (rl->aead == NULL)
+	if (rl->aead == NULL || content_type == SSL3_RT_CHANGE_CIPHER_SPEC)
 		return tls13_record_layer_seal_record_plaintext(rl,
 		    content_type, content, content_len);
 
@@ -774,12 +780,21 @@ tls13_record_layer_read_record(struct tls13_record_layer *rl)
 		if ((rl->rrec = tls13_record_new()) == NULL)
 			goto err;
 	}
-
-	if ((ret = tls13_record_recv(rl->rrec, rl->wire_read, rl->cb_arg)) <= 0)
+ 
+	if ((ret = tls13_record_recv(rl->rrec, rl->cb.wire_read, rl->cb_arg)) <= 0) {
+		switch (ret) {
+		case TLS13_IO_RECORD_VERSION:
+			return tls13_send_alert(rl, SSL_AD_PROTOCOL_VERSION);
+		case TLS13_IO_RECORD_OVERFLOW:
+			return tls13_send_alert(rl, SSL_AD_RECORD_OVERFLOW);
+		}
 		return ret;
-
-	/* XXX - record version checks. */
-
+	}
+ 
+	if (rl->legacy_version == TLS1_2_VERSION &&
+	    tls13_record_version(rl->rrec) != TLS1_2_VERSION)
+		return tls13_send_alert(rl, SSL_AD_PROTOCOL_VERSION);
+ 
 	content_type = tls13_record_content_type(rl->rrec);
 
 	/*
@@ -791,13 +806,13 @@ tls13_record_layer_read_record(struct tls13_record_layer *rl)
 	 */
 	if (content_type == SSL3_RT_CHANGE_CIPHER_SPEC) {
 		if (!rl->ccs_allowed || rl->ccs_seen >= 2)
-			return tls13_send_alert(rl, SSL_AD_UNEXPECTED_MESSAGE);
+			return tls13_send_alert(rl, TLS13_ALERT_UNEXPECTED_MESSAGE);
 		if (!tls13_record_content(rl->rrec, &cbs))
-			return tls13_send_alert(rl, TLS1_AD_DECODE_ERROR);
+			return tls13_send_alert(rl, TLS13_ALERT_DECODE_ERROR);
 		if (!CBS_get_u8(&cbs, &ccs))
-			return tls13_send_alert(rl, TLS1_AD_DECODE_ERROR);
+			return tls13_send_alert(rl, TLS13_ALERT_DECODE_ERROR);
 		if (ccs != 1)
-			return tls13_send_alert(rl, SSL_AD_ILLEGAL_PARAMETER);
+			return tls13_send_alert(rl, TLS13_ALERT_ILLEGAL_PARAMETER);
 		rl->ccs_seen++;
 		tls13_record_layer_rrec_free(rl);
 		return TLS13_IO_WANT_RETRY;
@@ -809,13 +824,23 @@ tls13_record_layer_read_record(struct tls13_record_layer *rl)
 	 * dummy ChangeCipherSpec messages, handled above).
 	 */
 	if (rl->aead != NULL && content_type != SSL3_RT_APPLICATION_DATA)
-		return tls13_send_alert(rl, SSL3_AD_UNEXPECTED_MESSAGE);
+		return tls13_send_alert(rl, TLS13_ALERT_UNEXPECTED_MESSAGE);
 
 	if (!tls13_record_layer_open_record(rl))
 		goto err;
 
 	tls13_record_layer_rrec_free(rl);
 
+	/*
+	 * On receiving a handshake or alert record with empty inner plaintext,
+	 * we must terminate the connection with an unexpected_message alert.
+	 * See RFC 8446 section 5.4.
+	 */
+	if (CBS_len(&rl->rbuf_cbs) == 0 &&
+	    (rl->rbuf_content_type == SSL3_RT_ALERT ||
+	     rl->rbuf_content_type == SSL3_RT_HANDSHAKE))
+		return tls13_send_alert(rl, TLS13_ALERT_UNEXPECTED_MESSAGE);
+
 	switch (rl->rbuf_content_type) {
 	case SSL3_RT_ALERT:
 		return tls13_record_layer_process_alert(rl);
@@ -825,11 +850,11 @@ tls13_record_layer_read_record(struct tls13_record_layer *rl)
 
 	case SSL3_RT_APPLICATION_DATA:
 		if (!rl->handshake_completed)
-			return tls13_send_alert(rl, SSL3_AD_UNEXPECTED_MESSAGE);
+			return tls13_send_alert(rl, TLS13_ALERT_UNEXPECTED_MESSAGE);
 		break;
 
 	default:
-		return tls13_send_alert(rl, SSL3_AD_UNEXPECTED_MESSAGE);
+		return tls13_send_alert(rl, TLS13_ALERT_UNEXPECTED_MESSAGE);
 	}
 
 	return TLS13_IO_SUCCESS;
@@ -864,7 +889,14 @@ tls13_record_layer_read_internal(struct tls13_record_layer *rl,
 		if ((ret = tls13_record_layer_read_record(rl)) <= 0)
 			return ret;
 
-		/* XXX - need to check record version. */
+		/*
+		 * We may have read a valid 0-byte application data record,
+		 * in which case we need to read the next record.
+		 */
+		if (CBS_len(&rl->rbuf_cbs) == 0) {
+			tls13_record_layer_rbuf_free(rl);
+			return TLS13_IO_WANT_POLLIN;
+		}
 	}
 
 	/*
@@ -872,7 +904,7 @@ tls13_record_layer_read_internal(struct tls13_record_layer *rl,
 	 * any record type that isn't a handshake until we are done.
 	 */
 	if (rl->phh && rl->rbuf_content_type != SSL3_RT_HANDSHAKE)
-		return tls13_send_alert(rl, SSL3_AD_UNEXPECTED_MESSAGE);
+		return tls13_send_alert(rl, TLS13_ALERT_UNEXPECTED_MESSAGE);
 
 	if (rl->rbuf_content_type != content_type) {
 		/*
@@ -903,8 +935,8 @@ tls13_record_layer_read_internal(struct tls13_record_layer *rl,
 				 *
 				 * TLS13_IO_FAILURE -> something broke.
 				 */
-				if (rl->phh_recv_cb != NULL) {
-					ret = rl->phh_recv_cb(
+				if (rl->cb.phh_recv != NULL) {
+					ret = rl->cb.phh_recv(
 					    rl->cb_arg, &rl->rbuf_cbs);
 				}
 
@@ -919,14 +951,18 @@ tls13_record_layer_read_internal(struct tls13_record_layer *rl,
 				 */
 				rl->phh = 0;
 
-				if (ret == TLS13_IO_SUCCESS)
-					return TLS13_IO_WANT_RETRY;
+				if (ret == TLS13_IO_SUCCESS) {
+					if (rl->phh_retry)
+						return TLS13_IO_WANT_RETRY;
+
+					return TLS13_IO_WANT_POLLIN;
+				}
 
 				return ret;
 			}
 		}
 
-		return tls13_send_alert(rl, SSL3_AD_UNEXPECTED_MESSAGE);
+		return tls13_send_alert(rl, TLS13_ALERT_UNEXPECTED_MESSAGE);
 	}
 
 	if (n > CBS_len(&rl->rbuf_cbs))
@@ -997,7 +1033,7 @@ tls13_record_layer_write_record(struct tls13_record_layer *rl,
 
 	/* See if there is an existing record and attempt to push it out... */
 	if (rl->wrec != NULL) {
-		if ((ret = tls13_record_send(rl->wrec, rl->wire_write,
+		if ((ret = tls13_record_send(rl->wrec, rl->cb.wire_write,
 		    rl->cb_arg)) <= 0)
 			return ret;
 		tls13_record_layer_wrec_free(rl);
@@ -1024,7 +1060,7 @@ tls13_record_layer_write_record(struct tls13_record_layer *rl,
 	if (!tls13_record_layer_seal_record(rl, content_type, content, content_len))
 		goto err;
 
-	if ((ret = tls13_record_send(rl->wrec, rl->wire_write, rl->cb_arg)) <= 0)
+	if ((ret = tls13_record_send(rl->wrec, rl->cb.wire_write, rl->cb_arg)) <= 0)
 		return ret;
 
 	tls13_record_layer_wrec_free(rl);
@@ -1064,6 +1100,25 @@ tls13_record_layer_write(struct tls13_record_layer *rl, uint8_t content_type,
 	return ret;
 }
 
+static const uint8_t tls13_dummy_ccs[] = { 0x01 };
+
+ssize_t
+tls13_send_dummy_ccs(struct tls13_record_layer *rl)
+{
+	ssize_t ret;
+
+	if (rl->ccs_sent)
+		return TLS13_IO_FAILURE;
+
+	if ((ret = tls13_record_layer_write(rl, SSL3_RT_CHANGE_CIPHER_SPEC,
+	    tls13_dummy_ccs, sizeof(tls13_dummy_ccs))) <= 0)
+		return ret;
+
+	rl->ccs_sent = 1;
+
+	return TLS13_IO_SUCCESS;
+}
+
 ssize_t
 tls13_read_handshake_data(struct tls13_record_layer *rl, uint8_t *buf, size_t n)
 {
@@ -1117,12 +1172,12 @@ tls13_write_application_data(struct tls13_record_layer *rl, const uint8_t *buf,
 ssize_t
 tls13_send_alert(struct tls13_record_layer *rl, uint8_t alert_desc)
 {
-	uint8_t alert_level = SSL3_AL_FATAL;
+	uint8_t alert_level = TLS13_ALERT_LEVEL_FATAL;
 	ssize_t ret;
 
-	if (alert_desc == SSL_AD_CLOSE_NOTIFY ||
-	    alert_desc == SSL_AD_USER_CANCELLED)
-		alert_level = SSL3_AL_WARNING;
+	if (alert_desc == TLS13_ALERT_CLOSE_NOTIFY ||
+	    alert_desc == TLS13_ALERT_USER_CANCELED)
+		alert_level = TLS13_ALERT_LEVEL_WARNING;
 
 	do {
 		ret = tls13_record_layer_alert(rl, alert_level, alert_desc);
