@@ -1,4 +1,4 @@
-/* $OpenBSD: tlsexttest.c,v 1.35.2.1 2020/08/10 18:59:47 tb Exp $ */
+/* $OpenBSD: tlsexttest.c,v 1.40 2020/05/24 20:01:58 tb Exp $ */
 /*
  * Copyright (c) 2017 Joel Sing <jsing@openbsd.org>
  * Copyright (c) 2017 Doug Hogan <doug@openbsd.org>
@@ -141,7 +141,7 @@ test_tlsext_alpn_client(void)
 
 	/* By default, we don't need this */
 	if (tlsext_alpn_client_needs(ssl)) {
-		FAIL("client should not need ALPN by default");
+		FAIL("client should not need ALPN by default\n");
 		goto err;
 	}
 
@@ -154,11 +154,11 @@ test_tlsext_alpn_client(void)
 	 */
 	if (SSL_set_alpn_protos(ssl, tlsext_alpn_single_proto_val,
 	    sizeof(tlsext_alpn_single_proto_val)) != 0) {
-		FAIL("should be able to set ALPN to http/1.1");
+		FAIL("should be able to set ALPN to http/1.1\n");
 		goto err;
 	}
 	if (!tlsext_alpn_client_needs(ssl)) {
-		FAIL("client should need ALPN by now");
+		FAIL("client should need ALPN by now\n");
 		goto err;
 	}
 
@@ -196,11 +196,11 @@ test_tlsext_alpn_client(void)
 	CBS_init(&cbs, tlsext_alpn_single_proto,
 	    sizeof(tlsext_alpn_single_proto));
 	if (!tlsext_alpn_server_parse(ssl, &cbs, &alert)) {
-		FAIL("failed to parse ALPN");
+		FAIL("failed to parse ALPN\n");
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -228,11 +228,11 @@ test_tlsext_alpn_client(void)
 
 	if (SSL_set_alpn_protos(ssl, tlsext_alpn_multiple_protos_val,
 	    sizeof(tlsext_alpn_multiple_protos_val)) != 0) {
-		FAIL("should be able to set ALPN to http/1.1");
+		FAIL("should be able to set ALPN to http/1.1\n");
 		goto err;
 	}
 	if (!tlsext_alpn_client_needs(ssl)) {
-		FAIL("client should need ALPN by now");
+		FAIL("client should need ALPN by now\n");
 		goto err;
 	}
 
@@ -263,11 +263,11 @@ test_tlsext_alpn_client(void)
 	CBS_init(&cbs, tlsext_alpn_multiple_protos,
 	    sizeof(tlsext_alpn_multiple_protos));
 	if (!tlsext_alpn_server_parse(ssl, &cbs, &alert)) {
-		FAIL("failed to parse ALPN");
+		FAIL("failed to parse ALPN\n");
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -298,7 +298,7 @@ test_tlsext_alpn_client(void)
 	ssl->internal->alpn_client_proto_list_len = 0;
 
 	if (tlsext_alpn_client_needs(ssl)) {
-		FAIL("client should need ALPN by default");
+		FAIL("client should need ALPN by default\n");
 		goto err;
 	}
 
@@ -360,7 +360,7 @@ test_tlsext_alpn_server(void)
 	/* Make sure we can build a server with one protocol */
 
 	if (!tlsext_alpn_server_build(ssl, &cbb)) {
-		FAIL("server should be able to build a response");
+		FAIL("server should be able to build a response\n");
 		goto err;
 	}
 	if (!CBB_finish(&cbb, &data, &dlen))
@@ -393,22 +393,22 @@ test_tlsext_alpn_server(void)
 
 	/* Shouldn't be able to parse without requesting */
 	if (tlsext_alpn_client_parse(ssl, &cbs, &alert)) {
-		FAIL("Should only parse server if we requested it");
+		FAIL("Should only parse server if we requested it\n");
 		goto err;
 	}
 
 	/* Should be able to parse once requested. */
 	if (SSL_set_alpn_protos(ssl, tlsext_alpn_single_proto_val,
 	    sizeof(tlsext_alpn_single_proto_val)) != 0) {
-		FAIL("should be able to set ALPN to http/1.1");
+		FAIL("should be able to set ALPN to http/1.1\n");
 		goto err;
 	}
 	if (!tlsext_alpn_server_parse(ssl, &cbs, &alert)) {
-		FAIL("Should be able to parse server when we request it");
+		FAIL("Should be able to parse server when we request it\n");
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -447,7 +447,7 @@ test_tlsext_alpn_server(void)
 	S3I(ssl)->alpn_selected_len = 0;
 
 	if (tlsext_alpn_server_needs(ssl)) {
-		FAIL("server should need ALPN by default");
+		FAIL("server should need ALPN by default\n");
 		goto err;
 	}
 
@@ -470,11 +470,10 @@ test_tlsext_alpn_server(void)
  */
 
 static uint8_t tlsext_supportedgroups_client_default[] = {
-	0x00, 0x08,
+	0x00, 0x06,
 	0x00, 0x1d,  /* X25519 (29) */
 	0x00, 0x17,  /* secp256r1 (23) */
-	0x00, 0x18,  /* secp384r1 (24) */
-	0x00, 0x19,  /* secp521r1 (25) */
+	0x00, 0x18   /* secp384r1 (24) */
 };
 
 static uint16_t tlsext_supportedgroups_client_secp384r1_val[] = {
@@ -612,7 +611,7 @@ test_tlsext_supportedgroups_client(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -712,7 +711,7 @@ test_tlsext_supportedgroups_client(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -917,7 +916,7 @@ test_tlsext_ecpf_client(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -1010,7 +1009,7 @@ test_tlsext_ecpf_client(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -1130,7 +1129,7 @@ test_tlsext_ecpf_server(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -1218,7 +1217,7 @@ test_tlsext_ecpf_server(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -1346,7 +1345,7 @@ test_tlsext_ri_client(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -1408,6 +1407,7 @@ test_tlsext_ri_server(void)
 	if ((ssl = SSL_new(ssl_ctx)) == NULL)
 		errx(1, "failed to create SSL");
 
+	ssl->version = TLS1_2_VERSION;
 	if (tlsext_ri_server_needs(ssl)) {
 		FAIL("server should not need RI\n");
 		goto err;
@@ -1459,7 +1459,7 @@ test_tlsext_ri_server(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -1579,7 +1579,7 @@ test_tlsext_sigalgs_client(void)
 		goto done;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto done;
 	}
 
@@ -1731,7 +1731,7 @@ test_tlsext_sni_client(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -1846,7 +1846,7 @@ test_tlsext_sni_server(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -1938,7 +1938,7 @@ test_tlsext_ocsp_client(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -2047,31 +2047,31 @@ test_tlsext_sessionticket_client(void)
 
 	/* Test disabling tickets. */
 	if ((SSL_set_options(ssl, SSL_OP_NO_TICKET) & SSL_OP_NO_TICKET) == 0) {
-		FAIL("Cannot disable tickets in the TLS connection");
+		FAIL("Cannot disable tickets in the TLS connection\n");
 		return 0;
 	}
 	if (tlsext_sessionticket_client_needs(ssl)) {
-		FAIL("client should not need SessionTicket if it was disabled");
+		FAIL("client should not need SessionTicket if it was disabled\n");
 		goto err;
 	}
 
 	/* Test re-enabling tickets. */
 	if ((SSL_clear_options(ssl, SSL_OP_NO_TICKET) & SSL_OP_NO_TICKET) != 0) {
-		FAIL("Cannot re-enable tickets in the TLS connection");
+		FAIL("Cannot re-enable tickets in the TLS connection\n");
 		return 0;
 	}
 	if (!tlsext_sessionticket_client_needs(ssl)) {
-		FAIL("client should need SessionTicket if it was disabled");
+		FAIL("client should need SessionTicket if it was disabled\n");
 		goto err;
 	}
 
 	/* Since we don't have a session, we should build an empty ticket. */
 	if (!tlsext_sessionticket_client_build(ssl, &cbb)) {
-		FAIL("Cannot build a ticket");
+		FAIL("Cannot build a ticket\n");
 		goto err;
 	}
 	if (!CBB_finish(&cbb, &data, &dlen)) {
-		FAIL("Cannot finish CBB");
+		FAIL("Cannot finish CBB\n");
 		goto err;
 	}
 	if (dlen != 0) {
@@ -2088,15 +2088,15 @@ test_tlsext_sessionticket_client(void)
 	if ((ssl->session = SSL_SESSION_new()) == NULL)
 		errx(1, "failed to create session");
 	if (!tlsext_sessionticket_client_needs(ssl)) {
-		FAIL("Should still want a session ticket with a new session");
+		FAIL("Should still want a session ticket with a new session\n");
 		goto err;
 	}
 	if (!tlsext_sessionticket_client_build(ssl, &cbb)) {
-		FAIL("Cannot build a ticket");
+		FAIL("Cannot build a ticket\n");
 		goto err;
 	}
 	if (!CBB_finish(&cbb, &data, &dlen)) {
-		FAIL("Cannot finish CBB");
+		FAIL("Cannot finish CBB\n");
 		goto err;
 	}
 	if (dlen != 0) {
@@ -2122,15 +2122,15 @@ test_tlsext_sessionticket_client(void)
 	ssl->session->tlsext_ticklen = sizeof(dummy);
 
 	if (!tlsext_sessionticket_client_needs(ssl)) {
-		FAIL("Should still want a session ticket with a new session");
+		FAIL("Should still want a session ticket with a new session\n");
 		goto err;
 	}
 	if (!tlsext_sessionticket_client_build(ssl, &cbb)) {
-		FAIL("Cannot build a ticket");
+		FAIL("Cannot build a ticket\n");
 		goto err;
 	}
 	if (!CBB_finish(&cbb, &data, &dlen)) {
-		FAIL("Cannot finish CBB");
+		FAIL("Cannot finish CBB\n");
 		goto err;
 	}
 	if (dlen != sizeof(dummy)) {
@@ -2157,12 +2157,12 @@ test_tlsext_sessionticket_client(void)
 	 * through SSL_set_options().
 	 */
 	if (!SSL_set_session_ticket_ext(ssl, NULL, 0)) {
-		FAIL("Could not set a NULL custom ticket");
+		FAIL("Could not set a NULL custom ticket\n");
 		goto err;
 	}
 	/* Should not need a ticket in this case */
 	if (tlsext_sessionticket_client_needs(ssl)) {
-		FAIL("Should not want to use session tickets with a NULL custom");
+		FAIL("Should not want to use session tickets with a NULL custom\n");
 		goto err;
 	}
 
@@ -2174,26 +2174,26 @@ test_tlsext_sessionticket_client(void)
 	ssl->internal->tlsext_session_ticket = NULL;
 
 	if (!tlsext_sessionticket_client_needs(ssl)) {
-		FAIL("Should need a session ticket again when the custom one is removed");
+		FAIL("Should need a session ticket again when the custom one is removed\n");
 		goto err;
 	}
 
 	/* Test a custom session ticket (not recommended in practice) */
 	if (!SSL_set_session_ticket_ext(ssl, tlsext_sessionticket_hello_max,
 	    sizeof(tlsext_sessionticket_hello_max))) {
-		FAIL("Should be able to set a custom ticket");
+		FAIL("Should be able to set a custom ticket\n");
 		goto err;
 	}
 	if (!tlsext_sessionticket_client_needs(ssl)) {
-		FAIL("Should need a session ticket again when the custom one is not empty");
+		FAIL("Should need a session ticket again when the custom one is not empty\n");
 		goto err;
 	}
 	if (!tlsext_sessionticket_client_build(ssl, &cbb)) {
-		FAIL("Cannot build a ticket with a max length random payload");
+		FAIL("Cannot build a ticket with a max length random payload\n");
 		goto err;
 	}
 	if (!CBB_finish(&cbb, &data, &dlen)) {
-		FAIL("Cannot finish CBB");
+		FAIL("Cannot finish CBB\n");
 		goto err;
 	}
 	if (dlen != sizeof(tlsext_sessionticket_hello_max)) {
@@ -2203,7 +2203,7 @@ test_tlsext_sessionticket_client(void)
 	}
 	if (memcmp(data, tlsext_sessionticket_hello_max,
 	    sizeof(tlsext_sessionticket_hello_max)) != 0) {
-		FAIL("Expected to get what we passed in");
+		FAIL("Expected to get what we passed in\n");
 		compare_data(data, dlen,
 		    tlsext_sessionticket_hello_max,
 		    sizeof(tlsext_sessionticket_hello_max));
@@ -2252,38 +2252,38 @@ test_tlsext_sessionticket_server(void)
 
 	/* Test disabling tickets. */
 	if ((SSL_set_options(ssl, SSL_OP_NO_TICKET) & SSL_OP_NO_TICKET) == 0) {
-		FAIL("Cannot disable tickets in the TLS connection");
+		FAIL("Cannot disable tickets in the TLS connection\n");
 		return 0;
 	}
 	if (tlsext_sessionticket_server_needs(ssl)) {
-		FAIL("server should not need SessionTicket if it was disabled");
+		FAIL("server should not need SessionTicket if it was disabled\n");
 		goto err;
 	}
 
 	/* Test re-enabling tickets. */
 	if ((SSL_clear_options(ssl, SSL_OP_NO_TICKET) & SSL_OP_NO_TICKET) != 0) {
-		FAIL("Cannot re-enable tickets in the TLS connection");
+		FAIL("Cannot re-enable tickets in the TLS connection\n");
 		return 0;
 	}
 	if (tlsext_sessionticket_server_needs(ssl)) {
-		FAIL("server should not need SessionTicket yet");
+		FAIL("server should not need SessionTicket yet\n");
 		goto err;
 	}
 
 	/* Set expected to require it. */
 	ssl->internal->tlsext_ticket_expected = 1;
 	if (!tlsext_sessionticket_server_needs(ssl)) {
-		FAIL("server should now be required for SessionTicket");
+		FAIL("server should now be required for SessionTicket\n");
 		goto err;
 	}
 
 	/* server hello's session ticket should always be 0 length payload. */
 	if (!tlsext_sessionticket_server_build(ssl, &cbb)) {
-		FAIL("Cannot build a ticket with a max length random payload");
+		FAIL("Cannot build a ticket with a max length random payload\n");
 		goto err;
 	}
 	if (!CBB_finish(&cbb, &data, &dlen)) {
-		FAIL("Cannot finish CBB");
+		FAIL("Cannot finish CBB\n");
 		goto err;
 	}
 	if (dlen != 0) {
@@ -2429,7 +2429,7 @@ test_tlsext_srtp_client(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -2496,7 +2496,7 @@ test_tlsext_srtp_client(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -2527,7 +2527,7 @@ test_tlsext_srtp_client(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -2556,7 +2556,7 @@ test_tlsext_srtp_client(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -2669,7 +2669,7 @@ test_tlsext_srtp_server(void)
 		goto err;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		goto err;
 	}
 
@@ -2715,13 +2715,13 @@ test_tlsext_srtp_server(void)
 #endif /* OPENSSL_NO_SRTP */
 
 unsigned char tlsext_clienthello_default[] = {
-	0x00, 0x34, 0x00, 0x0b, 0x00, 0x02, 0x01, 0x00,
-	0x00, 0x0a, 0x00, 0x0a, 0x00, 0x08, 0x00, 0x1d,
-	0x00, 0x17, 0x00, 0x18, 0x00, 0x19, 0x00, 0x23,
-	0x00, 0x00, 0x00, 0x0d, 0x00, 0x18, 0x00, 0x16,
-	0x08, 0x06, 0x06, 0x01, 0x06, 0x03, 0x08, 0x05,
-	0x05, 0x01, 0x05, 0x03, 0x08, 0x04, 0x04, 0x01,
-	0x04, 0x03, 0x02, 0x01, 0x02, 0x03,
+	0x00, 0x32, 0x00, 0x0b, 0x00, 0x02, 0x01, 0x00,
+	0x00, 0x0a, 0x00, 0x08, 0x00, 0x06, 0x00, 0x1d,
+	0x00, 0x17, 0x00, 0x18, 0x00, 0x23, 0x00, 0x00,
+	0x00, 0x0d, 0x00, 0x18, 0x00, 0x16, 0x08, 0x06,
+	0x06, 0x01, 0x06, 0x03, 0x08, 0x05, 0x05, 0x01,
+	0x05, 0x03, 0x08, 0x04, 0x04, 0x01, 0x04, 0x03,
+	0x02, 0x01, 0x02, 0x03,
 };
 
 unsigned char tlsext_clienthello_disabled[] = {
@@ -2781,7 +2781,7 @@ test_tlsext_clienthello_build(void)
 		goto err;
 	}
 	if ((SSL_set_options(ssl, SSL_OP_NO_TICKET) & SSL_OP_NO_TICKET) == 0) {
-		FAIL("failed to disable session tickets");
+		FAIL("failed to disable session tickets\n");
 		return 0;
 	}
 
@@ -2819,14 +2819,13 @@ test_tlsext_clienthello_build(void)
 }
 
 unsigned char tlsext_serverhello_default[] = {
-	0x00
+	0x00, 0x06, 0x00, 0x2b, 0x00, 0x02, 0x03, 0x04,
 };
-const size_t sizeof_tlsext_serverhello_default = 0;
 
 unsigned char tlsext_serverhello_enabled[] = {
-	0x00, 0x13, 0xff, 0x01, 0x00, 0x01, 0x00, 0x00,
-	0x05, 0x00, 0x00, 0x00, 0x0b, 0x00, 0x02, 0x01,
-	0x00, 0x00, 0x23, 0x00, 0x00,
+	0x00, 0x10, 0x00, 0x2b, 0x00, 0x02, 0x03, 0x04,
+	0x00, 0x0b, 0x00, 0x02, 0x01, 0x00, 0x00, 0x23,
+	0x00, 0x00,
 };
 
 static int
@@ -2861,18 +2860,18 @@ test_tlsext_serverhello_build(void)
 	if (!CBB_finish(&cbb, &data, &dlen))
 		errx(1, "failed to finish CBB");
 
-	if (dlen != sizeof_tlsext_serverhello_default) {
+	if (dlen != sizeof(tlsext_serverhello_default)) {
 		FAIL("got serverhello extensions with length %zu, "
 		    "want length %zu\n", dlen,
-		    sizeof_tlsext_serverhello_default);
+		    sizeof(tlsext_serverhello_default));
 		compare_data(data, dlen, tlsext_serverhello_default,
-		    sizeof_tlsext_serverhello_default);
+		    sizeof(tlsext_serverhello_default));
 		goto err;
 	}
 	if (memcmp(data, tlsext_serverhello_default, dlen) != 0) {
 		FAIL("serverhello extensions differs:\n");
 		compare_data(data, dlen, tlsext_serverhello_default,
-		    sizeof_tlsext_serverhello_default);
+		    sizeof(tlsext_serverhello_default));
 		goto err;
 	}
 
@@ -2993,7 +2992,7 @@ test_tlsext_versions_client(void)
 	}
 
 	if (!CBB_finish(&cbb, &data, &dlen)) {
-		FAIL("failed to finish CBB");
+		FAIL("failed to finish CBB\n");
 		failure = 1;
 		goto done;
 	}
@@ -3012,7 +3011,7 @@ test_tlsext_versions_client(void)
 		goto done;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		failure = 1;
 		goto done;
 	}
@@ -3068,7 +3067,7 @@ test_tlsext_versions_server(void)
 	}
 
 	if (!CBB_finish(&cbb, &data, &dlen)) {
-		FAIL("failed to finish CBB");
+		FAIL("failed to finish CBB\n");
 		failure = 1;
 		goto done;
 	}
@@ -3087,7 +3086,7 @@ test_tlsext_versions_server(void)
 		goto done;
 	}
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		failure = 1;
 		goto done;
 	}
@@ -3171,7 +3170,7 @@ test_tlsext_keyshare_client(void)
 	}
 
 	if (!CBB_finish(&cbb, &data, &dlen)) {
-		FAIL("failed to finish CBB");
+		FAIL("failed to finish CBB\n");
 		failure = 1;
 		goto done;
 	}
@@ -3193,7 +3192,7 @@ test_tlsext_keyshare_client(void)
 	}
 
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		failure = 1;
 		goto done;
 	}
@@ -3255,20 +3254,20 @@ test_tlsext_keyshare_server(void)
 	}
 
 	if (tls_extension_find(TLSEXT_TYPE_key_share, &idx) == NULL) {
-		FAIL("failed to find keyshare extension");
+		FAIL("failed to find keyshare extension\n");
 		failure = 1;
 		goto done;
 	}
 	S3I(ssl)->hs.extensions_seen |= (1 << idx);
 
 	if (!tlsext_keyshare_server_needs(ssl)) {
-		FAIL("server should need keyshare");
+		FAIL("server should need keyshare\n");
 		failure = 1;
 		goto done;
 	}
 
 	if (tlsext_keyshare_server_build(ssl, &cbb)) {
-		FAIL("server should not have built a keyshare response");
+		FAIL("server should not have built a keyshare response\n");
 		failure = 1;
 		goto done;
 	}
@@ -3282,19 +3281,19 @@ test_tlsext_keyshare_server(void)
 	CBS_init(&cbs, bogokey, sizeof(bogokey));
 	if (!tls13_key_share_peer_public(S3I(ssl)->hs_tls13.key_share,
 	    0x001d, &cbs)) {
-		FAIL("failed to load peer public key");
+		FAIL("failed to load peer public key\n");
 		failure = 1;
 		goto done;
 	}
 
 	if (!tlsext_keyshare_server_build(ssl, &cbb)) {
-		FAIL("server should be able to build a keyshare response");
+		FAIL("server should be able to build a keyshare response\n");
 		failure = 1;
 		goto done;
 	}
 
 	if (!CBB_finish(&cbb, &data, &dlen)) {
-		FAIL("failed to finish CBB");
+		FAIL("failed to finish CBB\n");
 		failure = 1;
 		goto done;
 	}
@@ -3321,7 +3320,7 @@ test_tlsext_keyshare_server(void)
 	}
 
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		failure = 1;
 		goto done;
 	}
@@ -3390,19 +3389,19 @@ test_tlsext_cookie_client(void)
 	S3I(ssl)->hs_tls13.cookie_len = strlen(cookie);
 
 	if (!tlsext_cookie_client_needs(ssl)) {
-		FAIL("client should need cookie");
+		FAIL("client should need cookie\n");
 		failure = 1;
 		goto done;
 	}
 
 	if (!tlsext_cookie_client_build(ssl, &cbb)) {
-		FAIL("client should have built a cookie response");
+		FAIL("client should have built a cookie response\n");
 		failure = 1;
 		goto done;
 	}
 
 	if (!CBB_finish(&cbb, &data, &dlen)) {
-		FAIL("failed to finish CBB");
+		FAIL("failed to finish CBB\n");
 		failure = 1;
 		goto done;
 	}
@@ -3425,7 +3424,7 @@ test_tlsext_cookie_client(void)
 	}
 
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		failure = 1;
 		goto done;
 	}
@@ -3485,19 +3484,19 @@ test_tlsext_cookie_server(void)
 	S3I(ssl)->hs_tls13.cookie_len = strlen(cookie);
 
 	if (!tlsext_cookie_server_needs(ssl)) {
-		FAIL("server should need cookie");
+		FAIL("server should need cookie\n");
 		failure = 1;
 		goto done;
 	}
 
 	if (!tlsext_cookie_server_build(ssl, &cbb)) {
-		FAIL("server have built a cookie response");
+		FAIL("server have built a cookie response\n");
 		failure = 1;
 		goto done;
 	}
 
 	if (!CBB_finish(&cbb, &data, &dlen)) {
-		FAIL("failed to finish CBB");
+		FAIL("failed to finish CBB\n");
 		failure = 1;
 		goto done;
 	}
@@ -3536,7 +3535,7 @@ test_tlsext_cookie_server(void)
 	}
 
 	if (CBS_len(&cbs) != 0) {
-		FAIL("extension data remaining");
+		FAIL("extension data remaining\n");
 		failure = 1;
 		goto done;
 	}
@@ -3550,6 +3549,85 @@ done:
 	return (failure);
 }
 
+unsigned char *valid_hostnames[] = {
+	"openbsd.org",
+	"op3nbsd.org",
+	"org",
+	"3openbsd.com",
+	"3-0penb-d.c-m",
+	"a",
+	"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.com",
+	"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa."
+	"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa."
+	"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa."
+	"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
+	NULL,
+};
+
+static int
+test_tlsext_valid_hostnames(void)
+{
+	int i, failure = 0;
+
+	for (i = 0; valid_hostnames[i] != NULL; i++) {
+		CBS cbs;
+		CBS_init(&cbs, valid_hostnames[i], strlen(valid_hostnames[i]));
+		if (!tlsext_sni_is_valid_hostname(&cbs)) {
+			FAIL("Valid hostname '%s' rejected\n",
+			    valid_hostnames[i]);
+			failure = 1;
+			goto done;
+		}
+	}
+ done:
+	return failure;
+}
+
+unsigned char *invalid_hostnames[] = {
+	"openbsd.org.",
+	"openbsd..org",
+	"openbsd.org-",
+	"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.com",
+	"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa."
+	"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa."
+	"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa."
+	"aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa.a",
+	"-p3nbsd.org",
+	"openbs-.org",
+	"openbsd\n.org",
+	"open_bsd.org",
+	"open\178bsd.org",
+	"open\255bsd.org",
+	NULL,
+};
+
+static int
+test_tlsext_invalid_hostnames(void)
+{
+	int i, failure = 0;
+	CBS cbs;
+
+	for (i = 0; invalid_hostnames[i] != NULL; i++) {
+		CBS_init(&cbs, invalid_hostnames[i],
+		    strlen(invalid_hostnames[i]));
+		if (tlsext_sni_is_valid_hostname(&cbs)) {
+			FAIL("Invalid hostname '%s' accepted\n",
+			    invalid_hostnames[i]);
+			failure = 1;
+			goto done;
+		}
+	}
+	CBS_init(&cbs, valid_hostnames[0],
+	    strlen(valid_hostnames[0]) + 1);
+	if (tlsext_sni_is_valid_hostname(&cbs)) {
+		FAIL("hostname with NUL byte accepted\n");
+		failure = 1;
+		goto done;
+	}
+ done:
+	return failure;
+}
+
 
 int
 main(int argc, char **argv)
@@ -3601,5 +3679,8 @@ main(int argc, char **argv)
 	failed |= test_tlsext_clienthello_build();
 	failed |= test_tlsext_serverhello_build();
 
+	failed |= test_tlsext_valid_hostnames();
+	failed |= test_tlsext_invalid_hostnames();
+
 	return (failed);
 }
